@@ -4,7 +4,8 @@ import pandas as pd
 import os
 import io
 
-from .decorators import add_data
+from .decorators import add_data, format_response
+import tally
 from .tally import Tally
 
 class DataSet:
@@ -96,6 +97,30 @@ class DataSet:
         self.qp_data = result['csv']
         self.dataset_type = 'quantipy'
 
+    def use_parquet(self, pq_data_filename, pq_meta_filename=None):
+
+        with open(pq_data_filename, mode='rb') as file:
+            fileContent_data = file.read()
+        
+        payload={}
+
+        files=[ 
+            ('pq', (pq_data_filename,io.BytesIO(fileContent_data), 'application/x-parquet')) 
+        ]
+
+        if pq_meta_filename is not None:
+            with open(pq_meta_filename, mode='rb') as file:
+                fileContent_meta = file.read()
+            files.append(
+                ('pq_meta',(pq_meta_filename,io.BytesIO(fileContent_meta),'text/plain')),
+            )
+        response = self.tally.post_request('tally', 'convert_data_to_csv_json', payload, files)
+        result = json.loads(response.content)
+        self.qp_meta = json.dumps(result['json'])
+        self.qp_data = result['csv']
+        self.dataset_type = 'quantipy'
+
+
     def use_unicom(self, mdd_filename, ddf_filename):
         with open(mdd_filename, mode='rb') as file:
             fileContent_mdd = file.read()
@@ -130,17 +155,25 @@ class DataSet:
         return (files, payload)
 
     @add_data
+    @format_response
     def crosstab(self, data_params=None, **kwargs):
         # initialise the payload with our chosen data
-        files, payload = self.prepare_post_params(data_params, kwargs)
-        response = self.tally.post_request('tally', 'crosstab', payload, files)
+        # construct a crosstab option which references the dataset we are uploading with the request
+        kwargs['dataset'] = 'one'
+        params = {"crosstabs":[kwargs]}
+        files, payload = self.prepare_post_params(data_params, params)
+        # the datasource will be a quantipy one, so we provide meta and data
+        datasources={"one":{"meta":payload.pop('meta'), "data":payload.pop('data')}}
+        payload['params']['datasources'] = datasources
+        response = self.tally.post_request('tally', 'joined_crosstab', payload, files)
         json_dict = json.loads(response.content)
         if 'result' in json_dict.keys():
-            df = Tally.result_to_dataframe(json_dict['result'])
-            return df
+            return json_dict
         else:
-            print(response.content)
-            raise ValueError(response.content)
+            if 'message' in json_dict.keys():
+                raise ValueError(json_dict['message'])
+            else:
+                raise ValueError("Crosstab returned no result.")
 
     @add_data
     def variables(self, data_params=None):
@@ -154,7 +187,7 @@ class DataSet:
         files, payload = self.prepare_post_params(data_params, kwargs)
         response = self.tally.post_request('tally', 'meta', payload, files)
         json_dict = json.loads(response.content)
-        return Tally.result_to_dataframe(json_dict)
+        return tally.result_to_dataframe(json_dict)
 
     @add_data
     def derive(self, data_params=None, **kwargs):
